<?php
namespace Geral\Repositories;

use \Fiscal\Entities\Notafiscal;
use \Geral\Entities\Prodfili;
use \ResultadoOperacional\Entities\Produtorop;


/**
 *
 */
class Itemproduto extends \Doctrine\ORM\EntityRepository
{

    /**
     * Verifica se existem algum produto com ou sem controle de estoque (verificado atraves do parametro
     * $utilizaControleEstoque) na nota fiscal.
     *
     * @param string $cdfilial
     * @param string $nrlanctonf
     * @param string $cdfilirece
     * @param int    $nrorg
     * @param string $utilizaControleEstoque
     *
     * @throws \Exception
     * @param int    $nrorg
     * @return boolean
     */
    public function verificaControleEstoque($cdfilial, $nrlanctonf, $cdfilirece, $nrorg,
                                            $utilizaControleEstoque = Prodfili::USA_CONTROLE_ESTOQUE)
    {
        $sql = "SELECT 1
                FROM DUAL
                WHERE EXISTS
                (
                   SELECT 1
                     FROM ITEMPRODUTO IP, PRODFILI PF, PRODUTO P
                     WHERE IP.CDFILIAL             = '$cdfilial'
                       AND IP.NRLANCTONF           = '$nrlanctonf'
                       AND PF.CDFILIAL             = '$cdfilirece'
                       AND IP.CDPRODUTO            = P.CDPRODUTO
                       AND PF.CDPRODUTO            = P.CDPRODESTO
                       AND NVL(PF.IDCNTRESTOQ, '" . Prodfili::USA_CONTROLE_ESTOQUE . "') = '$utilizaControleEstoque'
                       and nrorg                   = $nrorg
                )";

        try {
            return (bool)$this->getEntityManager()->getConnection()->fetchColumn($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function save($params){
        $sql = "INSERT INTO ITEMPRODUTO
                            (CDFILIAL,  CDPRODEMIS, VRUNITITEM, IDCONFORITEM, NRLANCTONF,
                            DSPRODEMIS, VRDESCITEM, CDFISOPEITEM, NRSEQITPROD, IDATUVRESTOQ,
                            VRTOTAITEM, CDSUFXCFOPPR, CDPRODUTO, QTITEMPROD, VRCONTITEM,
                            CDSITUTRIBPR, IDAVISOSITEM, VRBRUTITEM, CDFILDEST, NRAUTOFORN,
                            CDMARCA, DTPROGENTAF, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ,
                            NRSUBLOTE,CDFILIMOVI, VRDESPERITEM, CDOPERADOR, IDITEMORIGEM, IDORIGEMPROD)
                     VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        $connection = $this->getEntityManager()->getConnection();
        return $connection->executeUpdate($sql,$params);
    }

    public function somaTotalItensNF($nrlanctonf, $cdfilial)
    {
        $sql = <<<SQL
        SELECT DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(SUM(VRTOTAITEM), 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(SUM(VRTOTAITEM), 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(SUM(VRTOTAITEM), 2), '9999999999999.99'))) AS VRTOTALITENS,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(SUM(VRDESCITEM), 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(SUM(VRDESCITEM), 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(SUM(VRDESCITEM), 2), '9999999999999.99'))) AS VRTOTDESCITEM,
               SUM(VRTOTAITEM) AS VRTOTALITENSFLOAT,
               SUM(VRDESCITEM) AS VRTOTDESCITEMFLOAT
          FROM ITEMPRODUTO
         WHERE CDFILIAL = :CDFILIAL
           AND NRLANCTONF = :NRLANCTONF
SQL;
        $params = array(
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial
        );

        try {
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getSomaDescontoItem($nrlanctonf, $cdfilial)
    {
        $sql = <<<SQL
         SELECT SUM(VRDESCITEM) AS VRDESCITEMTOT
                  FROM ITEMPRODUTO
                 WHERE CDFILIAL = :CDFILIAL
                   AND NRLANCTONF = :NRLANCTONF
SQL;

        $params = array(
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial
        );

        try {
            return (float)$this->getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o valor total de despesa.
     *
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @param string $controleEstoque
     * @return float
     * @throws \Exception
     */
    public function getVrTotalDespesa(Notafiscal $notaFiscal, $controleEstoque = Prodfili::NAO_USA_CONTROLE_ESTOQUE)
    {
        $cdfilial   = $notaFiscal->getFilial()->getCdfilial();
        $cdtipooper = $notaFiscal->getEntrsaidop()->getCdtipooper();

        $mesEntrada = $notaFiscal->getDtentrsaid()->format("m");
        $anoEntrada = $notaFiscal->getDtentrsaid()->format("Y");
        // Captura o primeiro dia do mes.
        $dtinicial  = date(\Geral\Util\DateUtil::FORMATO_DATA, mktime(0, 0, 0, $mesEntrada, 1, $anoEntrada));
        // Captura o ultimo dia do mes.
        $dtfinal    = date("t/m/Y", mktime(0, 0, 0, $mesEntrada, 1, $anoEntrada));

        $sql = "SELECT SUM(VRTOTAITEM) VRDESPESA
                  FROM NOTAFISCAL N, ITEMPRODUTO I, TIPOOPERACROP T, PRODFILI PF, PRODUTO P,
                  (
                    SELECT DISTINCT A.CDPRODUTO
                      FROM PRODUTOROP R, PRODUTO A, PRODUTO B
                      WHERE (INSTR(A.CDARVPROD, B.CDARVPROD) = 1)
                        AND R.CDPRODUTO = B.CDPRODUTO
                        AND R.CDFILIAL = TIPOPARAM_DRO('$cdfilial')
                        AND IDTPPRODUTOROP = 'DE'
                  ) A
                  WHERE  N.CDFILIRECE    = '$cdfilial'
                    AND N.DTENTRSAID BETWEEN '$dtinicial' AND '$dtfinal'
                     AND N.IDENTRSAIDOP  = '" . \Geral\Entities\Tipooperac::OPERACAO_ENTRADA . "'
                     AND I.CDFILIAL      = N.CDFILIAL
                     AND I.NRLANCTONF    = N.NRLANCTONF
                     AND N.CDTIPOOPER    = T.CDTIPOOPER
                     AND N.IDENTRSAIDOP  = T.IDENTRSAIDOP
                     AND T.CDFILIAL      = TIPOPARAM_DRO('$cdfilial')
                     AND T.IDTPOPERACROP = '" .Produtorop::IMPOSTO_ORIGEM_DESPESAS_ENTRADA. "'
                     AND P.CDPRODUTO     = I.CDPRODUTO
                     AND I.CDPRODUTO     = A.CDPRODUTO
                     AND PF.CDFILIAL     = N.CDFILIRECE
                     AND PF.CDPRODUTO    = P.CDPRODESTO
                     AND NVL(PF.IDCNTRESTOQ,'" .Prodfili::USA_CONTROLE_ESTOQUE. "')   = '$controleEstoque'
                     AND T.CDTIPOOPER    = '$cdtipooper'";

        try {
            return (float)$this->getEntityManager()->getConnection()->fetchColumn($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o tipo de conta a pagar.
     *
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrorg
     * @return mixed
     * @throws \Exception
     */
    public function getTipoContaPagar($cdfilial, $nrlanctonf, $nrorg)
    {
        $sql = "SELECT TR.CDTPCTPAGAR
                 FROM (
                     SELECT MAX(P.CDARVPROD) AS CDARVPROD
                     FROM ITEMPRODUTO I, PRODUTO P
                     WHERE I.CDFILIAL    = '$cdfilial'
                       AND I.NRLANCTONF  = '$nrlanctonf'
                           AND I.NRORG       = $nrorg
                               AND I.CDPRODUTO   = P.CDPRODUTO
                    GROUP BY I.VRTOTAITEM
                    HAVING I.VRTOTAITEM = (
                       SELECT MAX(VRTOTAITEM)
                         FROM ITEMPRODUTO
                         WHERE CDFILIAL    = '$cdfilial'
                           AND NRLANCTONF  = '$nrlanctonf'
                     )
                 ) IT, PRODUTO PR, TIPOCTPAGAR TR
                 WHERE TR.CDTPCTPAGAR = PR.CDTPCTPAGAR
                   AND PR.CDARVPROD = (
                    SELECT MAX(P.CDARVPROD)
                      FROM TIPOCTRECEPROD C, PRODUTO P
                      WHERE C.CDPRODUTO = P.CDPRODUTO
                        AND (INSTR(IT.CDARVPROD, RTRIM(P.CDARVPROD)) = 1)
                 )";

        try {
            return $this->getEntityManager()->getConnection()->fetchColumn($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTipoContaPagarProd($cdfilial, $nrlanctonf, $nrorg){
        $sql = <<<SQL
         SELECT P3.CDPRODUTO, P3.CDARVPROD, P4.CDARVPROD AS CDNIVEL, P4.CDTPCTPAGAR,
               ROUND(A.VRBRUTITEM,2) AS VRBRUTITEM
          FROM ITEMPRODUTO A, PRODUTO P3, PRODUTO P4
         WHERE A.CDFILIAL = :P_CDFILIAL
           AND A.NRLANCTONF = :P_NRLANCTONF
           AND A.NRORG = :P_NRORG
           AND A.CDPRODUTO = P3.CDPRODUTO
           AND A.NRORG = P3.NRORG
           AND (INSTR(P3.CDARVPROD,NVL(RTRIM(P4.CDARVPROD),P3.CDARVPROD)) = 1)
           AND P4.CDTPCTPAGAR IS NOT NULL
           AND P4.CDARVPROD = (SELECT MAX(P1.CDARVPROD)
                         FROM ITEMPRODUTO IT,  PRODUTO P, PRODUTO P1
                        WHERE IT.CDFILIAL = A.CDFILIAL
                                  AND IT.NRLANCTONF = A.NRLANCTONF
                                  AND IT.CDPRODUTO = A.CDPRODUTO
                                  AND IT.NRORG = A.NRORG
                      AND IT.CDPRODUTO = P.CDPRODUTO
                      AND IT.NRORG = P.NRORG
                      AND (INSTR(P.CDARVPROD,NVL(RTRIM(P1.CDARVPROD),P.CDARVPROD)) = 1)
                      AND P1.CDTPCTPAGAR IS NOT NULL)
         ORDER BY P3.CDARVPROD
SQL;

        $params = array(
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDFILIAL" => $cdfilial,
            "P_NRORG" => $nrorg
        );

        try {
            return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o tipo de conta a receber.
     * @param $cdfilial
     * @param $nrlanctonf
     *
     * @return string
     */
    public function getTipoContaReceber ($cdfilial, $nrlanctonf)
    {
        $sql = "SELECT TR.CDTPCTRECEB
                  FROM (
                    SELECT MAX(CDARVPROD) AS CDARVPROD
                      FROM ITEMPRODUTO I, PRODUTO P
                      WHERE CDFILIAL    = '$cdfilial'
                        AND NRLANCTONF  = '$nrlanctonf'
                        AND I.CDPRODUTO = P.CDPRODUTO
                      GROUP BY VRTOTAITEM
                      HAVING VRTOTAITEM = (
                        SELECT MAX(VRTOTAITEM)
                          FROM ITEMPRODUTO
                          WHERE CDFILIAL   = '$cdfilial'
                            AND NRLANCTONF = '$nrlanctonf'
                      )
                  ) IT, PRODUTO PR , TIPOCTRECEBER TR
                  WHERE TR.CDTPCTRECEB = PR.CDTPCTRECEB
                    AND PR.CDARVPROD = (
                      SELECT MAX(P.CDARVPROD)
                        FROM TIPOCTRECEPROD C, PRODUTO P
                        WHERE C.CDPRODUTO = P.CDPRODUTO
                          AND (INSTR(IT.CDARVPROD, RTRIM(P.CDARVPROD)) = 1)
                    )";

        try {
            return $this->getEntityManager()->getConnection()->fetchColumn($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function consultaItemNotaFiscal($cdfilial, $nrlanctonf, $cdpais, $sgestado, $nrorg)
    {
        $sql = <<<SQL
        SELECT CDFILIAL, NRLANCTONF, NRSEQITPROD, CDCLASFISC AS NCM, NRORG,
               CDARVPROD, CDFISOPEITEM AS CFOP, CDSITUTRIBPR, IDSEGMENTOPROD,
               DTFABRPROD, CDBARPRODUTO, CDFISCOPERSN, CSOSN
          FROM (SELECT IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD, IT.CDCLASFISC, IT.NRORG,
                       IT.CDARVPROD, IT.CDFISOPEITEM, IT.CDSITUTRIBPR, IT.IDSEGMENTOPROD,
                       TO_CHAR(DTFABRPROD, 'YYYY-MM-DD') AS DTFABRPROD, IT.CDBARPRODUTO, IT.CDFISCOPERSN,
                       IT.CSOSN
                  FROM LOTE LT, (SELECT CDPRODUTO
                                   FROM VRMAXVENPROD
                                  WHERE CDPAIS = :CDPAISCLIE
                                    AND SGESTADO = :SGESTACLIE) VM,
                                (SELECT I.CDFILIAL, I.NRLANCTONF, I.NRSEQITPROD, PR.CDCLASFISC, I.NRORG,
                                        PR.CDARVPROD, CDFISOPEITEM, I.CDSITUTRIBPR, PR.IDSEGMENTOPROD,
                                        PR.CDBARPRODUTO, SUBSTR(I.CDFISCOPERSN, 2, 3) AS CDFISCOPERSN,
                                        I.CDPRODUTO, I.CDFILIMOVI, I.NRLOTEESTQ, I.NRSUBLOTE, I.CDPRODMOVI,
                                        I.CDFISCOPERSN AS CSOSN
                                   FROM ITEMPRODUTO I, PRODUTO PR
                                  WHERE I.CDFILIAL = :CDFILIAL
                                    AND I.NRLANCTONF = :NRLANCTONF
                                    AND I.NRORG     = :NRORG
                                    AND I.CDPRODUTO = PR.CDPRODUTO) IT
                 WHERE IT.CDPRODUTO = VM.CDPRODUTO(+)
                   AND IT.CDFILIMOVI = LT.CDFILIAL(+)
                   AND IT.NRORG       = LT.NRORG(+)
                   AND IT.NRLOTEESTQ = LT.NRLOTEESTQ(+)
                   AND IT.NRSUBLOTE  = LT.NRSUBLOTE(+)
                   AND IT.CDPRODMOVI = LT.CDPRODUTO(+))
         ORDER BY NRSEQITPROD

SQL;

        $params = array(
                    "CDFILIAL"   => $cdfilial,
                    "NRLANCTONF" => $nrlanctonf,
                    "CDPAISCLIE" => $cdpais,
                    "SGESTACLIE" => $sgestado,
                    "NRORG" => $nrorg
        );

        return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
    }

    public function consultaItemNFE($cdfilial, $nrlanctonf, $cdpais, $sgestado)
    {
        $sql = <<<SQL
        SELECT NRORG, CDFILIAL, NRLANCTONF, SUBSTR(NRSEQITPROD, 2, 3) as NrItem, CDPRODUTO, NRSEQITPROD,
               CDPRODEMIS, SUBSTR(DSPRODEMIS, 1, 120) AS DSPRODEMIS, CDBARPRODUTO,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(QTITEMPROD, 4), '99999999.9999')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(QTITEMPROD, 4), '99999999.9999')), LTRIM(TO_CHAR(ROUND(QTITEMPROD, 4), '99999999.9999'))) AS QTITEMPROD,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRUNITITEM, 10), '999999999999.9999999999')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRUNITITEM, 10), '999999999999.9999999999')), LTRIM(TO_CHAR(ROUND(VRUNITITEM, 10), '999999999999.9999999999'))) AS VRUNITITEM,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRFRETEITEM, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRFRETEITEM, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRFRETEITEM, 2), '9999999999999.99'))) AS VRFRETE,
               VRFRETEITEM,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRSEGURITEM, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRSEGURITEM, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRSEGURITEM, 2), '9999999999999.99'))) AS VRSEGURO,
               VRSEGURITEM,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRENCARGODESC, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRENCARGODESC, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRENCARGODESC, 2), '9999999999999.99'))) AS VRENCARGODESCONTO,
               VRENCARGODESC,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRDESCITEM, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRDESCITEM, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRDESCITEM, 2), '9999999999999.99'))) AS VRDESCONTO,
               VRDESCITEM,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRTOTAITEM, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRTOTAITEM, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRTOTAITEM, 2), '9999999999999.99'))) AS VRTOTAITEM,
               VRTOTAITEM AS VRTOTAITEMFLOAT,
               CDCLASFISC AS NCM,
               CDFISOPEITEM as CFOP,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRMAXVENDA, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRMAXVENDA, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRMAXVENDA, 2), '9999999999999.99'))) AS VRMAXVENDA,
               SGUNIDADE, NRLOTEESTQ,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(QTPRODLOTE, 3), '99999999.999')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(QTPRODLOTE, 3), '99999999.999')), LTRIM(TO_CHAR(ROUND(QTPRODLOTE, 3), '99999999.999'))) AS QTPRODLOTE,
               DTFABRPROD,
               DTVALILOTE,
               CDSITUTRIBPR,
               SUBSTR(CDSITUTRIBPR, 1, 1) as ORIGEM,
               SUBSTR(CDSITUTRIBPR, 2, 2) AS CST,
               IDSEGMENTOPROD,
               NRLOTEESTQ AS PRODUTO,
               CDARVPROD,
               CDFISCOPERSN,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VRDESPADUANEIRA, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VRDESPADUANEIRA, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VRDESPADUANEIRA, 2), '9999999999999.99'))) AS DESPESASADUANEIRAS,
               VRDESPADUANEIRA,
               DECODE(SUBSTR(LTRIM(TO_CHAR(ROUND(VROUTRASDESPITEM, 2), '9999999999999.99')), 1, 1), '.', '0'||LTRIM(TO_CHAR(ROUND(VROUTRASDESPITEM, 2), '9999999999999.99')), LTRIM(TO_CHAR(ROUND(VROUTRASDESPITEM, 2), '9999999999999.99'))) AS OUTRASDESPESAS,
               VROUTRASDESPITEM,
               NRPEDPROD,
               NRSEQPEDPROD
        FROM
        (SELECT IT.NRORG, IT.CDFILIAL, IT.NRLANCTONF, SUBSTR(IT.NRSEQITPROD, 2, 3) as NrItem, IT.CDPRODUTO, IT.NRSEQITPROD,
                IT.CDPRODEMIS, IT.DSPRODEMIS, IT.CDBARPRODUTO, IT.QTITEMPROD,
                IT.VRUNITITEM, IT.VRDESCITEM, IT.VRFRETEITEM, IT.VRTOTAITEM,
                IT.CDCLASFISC, IT.CDFISOPEITEM, VM.VRMAXVENDA, IT.SGUNIDADE,
                IT.NRLOTEESTQ, IT.QTITEMPROD AS QTPRODLOTE, TO_CHAR(DTFABRPROD, 'YYYY-MM-DD') AS DTFABRPROD, IT.VRSEGURITEM,
                TO_CHAR(DTVALILOTE, 'YYYY-MM-DD') AS DTVALILOTE, IT.VRENCARGODESC,
                IT.CDSITUTRIBPR, IT.IDSEGMENTOPROD, IT.NRLOTEESTQ AS PRODUTO,
                IT.CDARVPROD, SUBSTR(IT.CDFISCOPERSN, 2, 3) AS CDFISCOPERSN, IT.VRDESPADUANEIRA, IT.VROUTRASDESPITEM,
                IT.NRPEDPROD, IT.NRSEQPEDPROD
          FROM LOTE LT,
             (SELECT NRORG, CDPAIS, SGESTADO, CDPRODUTO, VRMAXVENDA,
                     VRPEMAXVENDA
                FROM VRMAXVENPROD
               WHERE CDPAIS = :CDPAISCLIE
                 AND SGESTADO = :SGESTACLIE) VM,
             (SELECT I.NRORG, I.CDFILIAL, I.NRLANCTONF, SUBSTR(I.NRSEQITPROD, 2, 3) AS NRITEM, I.CDPRODUTO, I.NRSEQITPROD,
                     I.CDPRODEMIS, I.DSPRODEMIS, PR.CDBARPRODUTO, I.QTITEMPROD,
                     I.VRUNITITEM, I.VRDESCITEM, I.VRFRETEITEM, I.VRTOTAITEM,
                     PR.CDCLASFISC, I.CDFISOPEITEM, PR.SGUNIDADE, I.VRSEGURITEM,
                     I.NRLOTEESTQ, I.QTITEMPROD AS QTPRODLOTE, I.VRENCARGODESC,
                     I.CDSITUTRIBPR, PR.IDSEGMENTOPROD, I.NRLOTEESTQ AS PRODUTO,
                     PR.CDARVPROD, I.CDFILIMOVI, I.NRSUBLOTE, NVL(I.CDPRODMOVI, PR.CDPRODESTO) AS CDPRODMOVI,
                     I.CDFISCOPERSN, I.VRDESPADUANEIRA, I.VROUTRASDESPITEM, I.NRPEDPROD, I.NRSEQPEDPROD
                FROM ITEMPRODUTO I, PRODUTO PR
               WHERE I.CDFILIAL = :CDFILIAL
                 AND I.NRLANCTONF = :NRLANCTONF
                 AND I.CDPRODUTO = PR.CDPRODUTO
                 AND I.NRORG     = PR.NRORG) IT
         WHERE IT.CDPRODUTO = VM.CDPRODUTO(+)
           AND IT.NRORG     = VM.NRORG(+)
           AND IT.CDFILIMOVI = LT.CDFILIAL(+)
           AND IT.NRLOTEESTQ = LT.NRLOTEESTQ(+)
           AND IT.NRSUBLOTE  = LT.NRSUBLOTE(+)
           AND IT.CDPRODMOVI = LT.CDPRODUTO(+)
           AND IT.NRORG      = LT.NRORG(+))
        ORDER BY NRSEQITPROD
SQL;

        $params = array(
            "CDPAISCLIE" => $cdpais,
            "SGESTACLIE" => $sgestado,
            "CDFILIAL"   => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );

        try{
//            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function consultaListaNFE($cdfilial, $nrlanctonf)
    {
        $sql = <<<SQL
        SELECT RPAD(NMGRUPO,16, ' ')||' R$: '||REPLACE(TO_CHAR(ROUND(VRLISTA,2),'99999999.00'),'.', ',') AS DSLISTA
        FROM(
        SELECT SUBSTR(P.CDARVPROD, 1, 1) AS GRUPO, 'Lista Positiva' AS NMGRUPO, SUM(VRTOTAITEM)AS VRLISTA
        FROM ITEMPRODUTO I, PRODUTO P
        WHERE I.CDFILIAL   = :FILIAL   AND
              I.NRLANCTONF = :LANCAMENTO AND
              I.CDPRODUTO  = P.CDPRODUTO AND
              I.NRORG      = P.NRORG     AND
              P.IDSEGMENTOPROD = 'MEDI' AND
              (SUBSTR(P.CDARVPROD, 1, 5) = '50101' OR SUBSTR(P.CDARVPROD, 1, 5) = '50103' OR SUBSTR(P.CDARVPROD, 1, 5) = '50301' OR SUBSTR(P.CDARVPROD, 1, 5) = '50303')
        GROUP BY SUBSTR(P.CDARVPROD, 1, 1)
        UNION ALL
        SELECT SUBSTR(P.CDARVPROD, 1, 1) AS GRUPO, 'Lista Negativa' AS NMGRUPO, SUM(VRTOTAITEM)AS VRLISTA
        FROM ITEMPRODUTO I, PRODUTO P
        WHERE I.CDFILIAL     = :FILIAL   AND
              I.NRLANCTONF   = :LANCAMENTO AND
              I.CDPRODUTO = P.CDPRODUTO AND
              I.NRORG     = P.NRORG     AND
              P.IDSEGMENTOPROD = 'MEDI' AND
              (SUBSTR(P.CDARVPROD, 1, 5) = '50102' OR SUBSTR(P.CDARVPROD, 1, 5) = '50104' OR SUBSTR(P.CDARVPROD, 1, 5) = '50302' OR SUBSTR(P.CDARVPROD, 1, 5) = '50304')
        GROUP BY SUBSTR(P.CDARVPROD, 1, 1))
SQL;

        $params = array(
            "FILIAL" => $cdfilial,
            "LANCAMENTO" => $nrlanctonf,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getMaxNRSEQITPROD($filial, $lancamento){
        $sql = <<<SQL
        SELECT NVL(MAX(NRSEQITPROD),'0000') AS NRSEQITPROD
          FROM ITEMPRODUTO
         WHERE CDFILIAL   = :P_CDFILIAL
           AND NRLANCTONF = :P_NRLANCTONF
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getMaxNRSEQUITNOTA($filial, $lancamento){
    	$sql = <<<SQL
        SELECT NVL(MAX(NRSEQUITNOTA),'000') AS NRSEQUITNOTA
          FROM ITEMNOTAFISC
         WHERE CDFILIAL   = :P_CDFILIAL
           AND NRLANCTONF = :P_NRLANCTONF
SQL;
    
    	$params = array(
    			"P_CDFILIAL" => $filial,
    			"P_NRLANCTONF" => $lancamento,
    	);
    
    	try{
    		return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
    	} catch(\Exception $e) {
    		throw $e;
    	}
    }

    public function getMaxCDFISOPEITEM($filial, $lancamento, $nrorg){
        $sql = <<<SQL
        SELECT MAX(CDFISOPEITEM)
          FROM ITEMPRODUTO
         WHERE CDFILIAL   = :P_CDFILIAL
           AND NRLANCTONF = :P_NRLANCTONF
           AND NRORG      = :P_NRORG
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_NRORG" => $nrorg
        );

        try{
            return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function consultaQuantidadeJaDevolvida($lancamento, $filialOrigem, $lancamentoOrigem, $nrdecitotanf, $produto, $cdarvprod){
    	$sql = <<<SQL
        SELECT NVL(SUM(I.QTITEMPROD),0) AS QTITEMDEVO
		  FROM ITEMPRODUTO I, PRODUTO P,
		     (SELECT DISTINCT B.CDFILIAL, B.NRLANCTONF
		        FROM ITEMPRODUTO B, ITEMNOTAFISC I
		       WHERE (I.CDFILIALIT   = :P_CDFILIORIG)
		         AND (I.NRLANCTONFIT = :P_NRLANCTONFORIG)
		         AND (B.CDFILIAL     = I.CDFILIAL)
		         AND (B.NRLANCTONF   = I.NRLANCTONF)) O
		 WHERE I.CDFILIAL = O.CDFILIAL
		   AND I.NRLANCTONF = O.NRLANCTONF
		   AND (P.CDARVPROD    = :P_CDARVPROD)
		   AND (I.CDPRODUTO    = P.CDPRODUTO)
		   AND (NVL(I.NRLOTEESTQ, ' ')   = NVL(:P_NRLOTEESTQ, ' '))
		   AND (NVL(I.NRSUBLOTE, ' ')    = NVL(:P_NRSUBLOTE, ' '))
		   AND (ROUND(I.VRUNITITEM,:P_NRDECITOTANF) = ROUND(:P_VRUNITITEM,:P_NRDECITOTANF))
		   AND (I.CDFILIAL || I.NRLANCTONF) <> (:P_CDFILIAL || :P_NRLANCTONF)
SQL;
    
    	$params = array(
    			"P_CDFILIORIG" => $filialOrigem,
    			"P_NRLANCTONFORIG" => $lancamentoOrigem,
    			"P_CDARVPROD" => $cdarvprod,
    			"P_NRLOTEESTQ" => $produto->getNrloteestq(),
    			"P_NRSUBLOTE" => $produto->getNrsublote(),
    			"P_NRDECITOTANF" => $nrdecitotanf,
    			"P_VRUNITITEM" => $produto->getVrunititem(),
    			"P_CDFILIAL" => $produto->getCdfilial(),
    			"P_NRLANCTONF" => $lancamento,
    	);
    
    	try{
    		return self::getEntityManager()->getConnection()->fetchColumn($sql, $params);
    	} catch(\Exception $e) {
    		throw $e;
    	}
    }

    /*
     * Verifica o valor bruto dos itens que não irão compor o vencimento da nota fiscal
     */
    public function getVrSubtraiVencimento($cdfilial, $nrlanctonf, $nrorg, $cdtipooper){
        $sql = <<<SQL
        SELECT NVL(SUM(VRBRUTITEM),0) AS VRSUBTRAIVENC
           FROM ITEMPRODUTO IT, TPOPERCFOP T
          WHERE IT.CDFILIAL   = :P_CDFILIAL
            AND IT.NRLANCTONF = :P_NRLANCTONF
            AND IT.NRORG      = :P_NRORG
            AND T.IDENTRSAIDOP = 'S'
            AND T.CDTIPOOPER   = :P_CDTIPOOPER
            AND T.IDCOMPOEVENC = 'N'
            AND T.CDFISCOPER   = IT.CDFISOPEITEM
SQL;

        $params = array(
            'P_CDFILIAL'      => $cdfilial,
            'P_NRLANCTONF'   => $nrlanctonf,
            'P_NRORG'      => $nrorg,
            'P_CDTIPOOPER'      => $cdtipooper
        );

        return $this->getEntityManager()->getConnection()->fetchColumn($sql, $params);
    }

    public function getNFItemProd($nrorg, $cdfilial, $nrpedido){
        try{
            $sql = <<<SQL
                    SELECT DISTINCT I.NRLANCTONF
                      FROM ITEMPRODUTO I, NOTAFISCAL N
                     WHERE I.NRORG      = N.NRORG
                       AND I.CDFILIAL   = N.CDFILIAL
                       AND I.NRLANCTONF = N.NRLANCTONF
                       AND I.NRPEDIDO   = :P_NRPEDIDO
                       AND I.CDFILIAL   = :P_CDFILIAL
                       AND I.NRORG      = :P_NRORG
SQL;

            $params = array(
                'P_NRORG'    => $nrorg,
                'P_CDFILIAL' => $cdfilial,
                'P_NRPEDIDO' => $nrpedido
            );

            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function buscaQuantidadeDevolvida($filialLancamento, $lancamento, $filialOrigem, $lancamentoOrigem, $codArvProd, $lote, $subLote, $valorUnitario, $organizacao){
        $sql = <<<SQL
        SELECT NVL(SUM(I.QTITEMPROD),0) AS QTITEMDEVO
          FROM ITEMPRODUTO I, PRODUTO P,
             (SELECT DISTINCT B.CDFILIAL, B.NRLANCTONF
                FROM ITEMPRODUTO B,
                    (SELECT CDFILIAL, NRLANCTONF
                       FROM ITEMNOTAFISC
                      WHERE CDFILIALIT = :P_CDFILIALIT
                        AND NRLANCTONFIT = :P_NRLANCTONFIT) L
               WHERE (B.CDFILIAL     = L.CDFILIAL)
                 AND (B.NRLANCTONF   = L.NRLANCTONF)) O,
             (SELECT  NVL(NRDECITOTANF, 2) NRDECITOTANF, NVL(NRDUPLINFS, 0) AS NRDUPLINFS,
                      NVL(IDUTCTRLFISSAID, 'N') AS IDUTCTRLFISSAID, NVL(IDPERVISIMPSCRE, 'S') AS IDPERVISIMPSCRE
                FROM PARAMGERAL
               WHERE PARAMGERAL.NRORG = :P_NRORG) PG
         WHERE I.CDFILIAL = O.CDFILIAL
           AND I.NRLANCTONF = O.NRLANCTONF
           AND (P.CDARVPROD    = :P_CDARVPROD)
           AND (I.CDPRODUTO    = P.CDPRODUTO)
           AND (NVL(I.NRLOTEESTQ, ' ')   = NVL(:P_NRLOTEESTQ, ' '))
           AND (NVL(I.NRSUBLOTE, ' ')    = NVL(:P_NRSUBLOTE, ' '))
           AND (ROUND(I.VRUNITITEM, PG.NRDECITOTANF) = ROUND(:P_VRUNITITEM, PG.NRDECITOTANF))
           AND (I.CDFILIAL || I.NRLANCTONF) <> (:P_FILIAL || :P_LANCTONF)
SQL;

        $params = array(
            "P_CDFILIALIT" => $filialOrigem,
            "P_NRLANCTONFIT" => $lancamentoOrigem,
            "P_CDARVPROD" => $codArvProd,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $subLote,
            "P_VRUNITITEM" => $valorUnitario,
            "P_FILIAL" => $filialLancamento,
            "P_LANCTONF" => $lancamento,
            "P_NRORG" => $organizacao
        );

        try {
            return $this->getEntityManager()->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getCdmarcaItemProd($filial,$numeroNota,$nrseqitprod){
      $sql = <<<SQL
        SELECT CDMARCA
          FROM ITEMPRODUTO
        WHERE CDFILIAL  = :P_CDFILIAL
          AND NRLANCTONF  = :P_NRLANCTONF
          AND NRSEQITPROD = :P_NRSEQITPROD
SQL;

      $params = array(
        "P_CDFILIAL" => $filial,
        "P_NRLANCTONF" => $numeroNota,
        "P_NRSEQITPROD" => $nrseqitprod
      );

      try {
        return $this->getEntityManager()->getConnection()->fetchAssoc($sql, $params);
      } catch(\Exception $e) {
        throw $e;
      }
    }
}
