/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8350_SUPVIE_PRODACOMPRAR.SQL                                        */
/*                                                                            */
/* Descrio:                                                                 */
/* Agrupa todos os produtos a comprar solicitados (Solicitacao Compras)       */
/* e que ainda no possuem foram atendidos (AF).                              */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 04/05/2000 - Yoris L. de Souza                                             */
/*                                                                            */
/* Manutenes:                                                               */
/*  19/05/2000 - Converso para SqlServer                                     */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'PRODACOMPRAR'  and
            Xtype = 'V'                  and
            u.uid = o.uid               and
            u.name = USER)
	  DROP VIEW PRODACOMPRAR;
GO

CREATE VIEW PRODACOMPRAR AS
SELECT
   COMP.CDFILIAL, COMP.DTPROGENTR,
   COMP.CDPRODUTO,
   ROUND(COMP.QTDECOMP-ISNULL(JACOMP.QTDECOMP/COMP.VRFATOCONV,0), 5) AS QTACOMPRAR
FROM
   PARASUPRI,
   FILIAL,
   (SELECT
     ITEMSOLI.CDFILSOLI AS CDFILIAL,
     ITEMSOLI.DTPROGENTR,
     ITEMSOLI.CDPRODUTO,
     PRODUTO.CDPRODESTO,
     PRODUTO.VRFATOCONV,
     SUM(ITEMSOLI.QTSOLIPROD) AS QTDECOMP
    FROM
     PRODUTO, ITEMSOLI
    WHERE
     (ITEMSOLI.IDITMCONS = 'N') AND
     (ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO) AND
     (ISNULL(ITEMSOLI.IDSOLIAPROV,'S') = 'S')
    GROUP BY
     ITEMSOLI.CDFILSOLI,
     ITEMSOLI.DTPROGENTR,
     PRODUTO.CDPRODESTO,
     ITEMSOLI.CDPRODUTO,
     PRODUTO.VRFATOCONV) COMP LEFT JOIN
   (SELECT
     ITEMAUTF.CDFILDEST AS CDFILIAL,
     ITEMAUTF.DTENTRITMSOL,
     PRODUTO.CDPRODESTO,
     SUM(ITEMAUTF.QTPROGENTAF*PRODUTO.VRFATOCONV) AS QTDECOMP
    FROM
     PRODUTO, ITEMAUTF, AUTOFORNE
    WHERE
    (AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST) AND
    (AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC) AND
    (AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN) AND
    (ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO) AND
    (AUTOFORNE.IDSITUATUAAF IS NULL) AND
    (AUTOFORNE.IDAFFORALIM = 'N') AND
    (AUTOFORNE.IDAUTOFORNEX = 'N') AND
    (ISNULL(AUTOFORNE.IDBAIXAAF,'0') <> '5')
   GROUP BY
    ITEMAUTF.CDFILDEST,
    ITEMAUTF.DTENTRITMSOL,
    PRODUTO.CDPRODESTO) JACOMP
   ON
    COMP.CDFILIAL = JACOMP.CDFILIAL AND
    COMP.DTPROGENTR = JACOMP.DTENTRITMSOL AND
    COMP.CDPRODESTO = JACOMP.CDPRODESTO 
WHERE
   FILIAL.CDFILIAL = COMP.CDFILIAL AND
   PARASUPRI.CDFILIAL = ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AND
   ISNULL(PARASUPRI.IDCONSPRSOL, 'N') = 'N' AND

   ROUND(COMP.QTDECOMP-ISNULL(JACOMP.QTDECOMP/COMP.VRFATOCONV,0), 5) > 0

UNION ALL

SELECT
   COMP.CDFILIAL, COMP.DTPROGENTR,
   COMP.CDPRODUTO,
   ROUND(COMP.QTDECOMP-ISNULL(JACOMP.QTDECOMP,0), 5) AS QTACOMPRAR
FROM
   PARASUPRI,
   FILIAL,
   (SELECT
     ITEMSOLI.CDFILSOLI AS CDFILIAL,
     ITEMSOLI.DTPROGENTR,
     ITEMSOLI.CDPRODUTO,
     SUM(ITEMSOLI.QTSOLIPROD) AS QTDECOMP
    FROM
     ITEMSOLI
    WHERE
     (ITEMSOLI.IDITMCONS = 'N') AND
     (ISNULL(ITEMSOLI.IDSOLIAPROV,'S') = 'S')
    GROUP BY
     ITEMSOLI.CDFILSOLI,
     ITEMSOLI.DTPROGENTR,
     ITEMSOLI.CDPRODUTO) COMP LEFT JOIN
   (SELECT
     ITEMAUTF.CDFILDEST AS CDFILIAL,
     ITEMAUTF.DTENTRITMSOL,
     ITEMAUTF.CDPRODUTO,
     SUM(ITEMAUTF.QTPROGENTAF) AS QTDECOMP
    FROM
     ITEMAUTF, AUTOFORNE
    WHERE
    (AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST) AND
    (AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC) AND
    (AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN) AND
    (AUTOFORNE.IDSITUATUAAF IS NULL) AND
    (AUTOFORNE.IDAFFORALIM = 'N') AND
    (AUTOFORNE.IDAUTOFORNEX = 'N') AND
    (ISNULL(AUTOFORNE.IDBAIXAAF,'0') <> '5')
   GROUP BY
    ITEMAUTF.CDFILDEST,
    ITEMAUTF.DTENTRITMSOL,
    ITEMAUTF.CDPRODUTO) JACOMP
   ON
    COMP.CDFILIAL = JACOMP.CDFILIAL AND
    COMP.DTPROGENTR = JACOMP.DTENTRITMSOL AND
    COMP.CDPRODUTO = JACOMP.CDPRODUTO 
WHERE
   FILIAL.CDFILIAL = COMP.CDFILIAL AND
   PARASUPRI.CDFILIAL = ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AND
   PARASUPRI.IDCONSPRSOL = 'S' AND

   ROUND(COMP.QTDECOMP-ISNULL(JACOMP.QTDECOMP,0), 5) > 0
GO
