import {
  Metadata, Loader, Http, IDictionary, DatasourceFactory, IEventParam,
} from '@zeedhi/core';
import {
  Tabs, TreeGridEditable, Form, Grid, GridEditable, Button, IComponentRender, DialogService, ModalService, Modal,
} from '@zeedhi/common';

export class Pla01300ModoPreparoController {
  private appController = Loader.getInstance('AppController');
  
  private utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public crudController = Loader.getInstance('CrudController');
  
  private cardModoPreparo = false;
  
  public newRow = false;
  
  public modalAddModoPrep!: Modal;
  
  public buttonDisable() {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    let form: any = '';
    let row: any = '';
    let grid: any = '';
    let button: any = '';

    if (tab.activeTab === 3) {
      form = Metadata.getInstance<Form>('FORM_TIPOCARD');
      row = form.value;
      grid = Metadata.getInstance<Grid>('gridModoPreparoTipoCard');
      button = grid.getChildInstance('addButton');
      
      if (!row.TIPOCARD?.CDTIPOCARD) {
        button.disabled = true;
        grid.datasource.data = [];
      } else button.disabled = false;
    } else if (tab.activeTab === 4) {
      form = Metadata.getInstance<Form>('FORM_LOCAL');
      row = form.value;
      grid = Metadata.getInstance<Grid>('gridModoPreparoLocal');
      button = grid.getChildInstance('addButton');
      
      if (!row.BRANCH_LOCAL?.CDFILIAL) {
        button.disabled = true;
        grid.datasource.data = [];
      } else button.disabled = false;
    } else if (tab.activeTab === 5) {
      form = Metadata.getInstance<Form>('FORM_SERVICE');
      row = form.value;
      grid = Metadata.getInstance<Grid>('gridModoPreparoServico');
      button = grid.getChildInstance('addButton');
      
      if (!row.BRANCH_SERVICE?.CDFILIAL || !row.SERVICE?.CDSERVFILI) {
        button.disabled = true;
        grid.datasource.data = [];
      } else button.disabled = false;
    }
  }
  
  public setValuesCurrentForm() {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    let grid: any = '';
    
    if (tab.activeTab === 2) grid = Metadata.getInstance<Grid>('gridModoPreparoPadrao');
    else if (tab.activeTab === 3) grid = Metadata.getInstance<Grid>('gridModoPreparoTipoCard');
    else if (tab.activeTab === 4) grid = Metadata.getInstance<Grid>('gridModoPreparoLocal');
    else if (tab.activeTab === 5) grid = Metadata.getInstance<Grid>('gridModoPreparoServico');
    
    if (grid.datasource.currentRow) {
      const formAdd = Metadata.getInstance<Form>('formAddModoPreparo');
      const row = formAdd.value;

      if (tab.activeTab === 2) {
        row.NRPREP = grid.datasource.currentRow.NRPREPPRAT;
        row.IDPREP = grid.datasource.currentRow.IDPREPPRAT;
        row.NMRPREP = grid.datasource.currentRow.NMRPREPPRAT;
        row.TXPREP = grid.datasource.currentRow.TXPREPPRAT;
      } else if (tab.activeTab === 3) {
        row.NRPREP = grid.datasource.currentRow.NRPREPTPCARD;
        row.IDPREP = grid.datasource.currentRow.IDPREPTPCARD;
        row.NMRPREP = grid.datasource.currentRow.NMRPREPTPCARD;
        row.TXPREP = grid.datasource.currentRow.TXPREPTPCARD;
      } else if (tab.activeTab === 4) {
        row.NRPREP = grid.datasource.currentRow.NRPREPFILI;
        row.IDPREP = grid.datasource.currentRow.IDPREPFILI;
        row.NMRPREP = grid.datasource.currentRow.NMRPREPFILI;
        row.TXPREP = grid.datasource.currentRow.TXPREPFILI;
      } else if (tab.activeTab === 5) {
        row.NRPREP = grid.datasource.currentRow.NRPREPSERV;
        row.IDPREP = grid.datasource.currentRow.IDPREPSERV;
        row.NMRPREP = grid.datasource.currentRow.NMRPREPSERV;
        row.TXPREP = grid.datasource.currentRow.TXPREPSERV;
      }
    }
  }
  
  public changeOverflowSelect() {
    const grid: Grid = Metadata.getInstance('gridModoPreparoPadrao');
    grid.columns[2].overflow = 2;
    this.setFilters();
  }
  
  private onChangeModoPreparo() {
    this.addRow();
  }
  
  public defaultState(grid: GridEditable, cloneDs: any) {
    this.newRow = false;
    return grid.cancelEditedRows().then(() => {
      this.gridEditController.onInlineEdit({ component: grid });
      return cloneDs.get();
    });
  }
    
  public async setFilters() {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    let grid: any = '';
    let form: any = '';
    let conditions: any = [];
    
    if (tab.activeTab === 2) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoPadrao');
      conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: currentRow.CDPRATO,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: currentRow.CDFILAUXPRAT,
        },
      ];
      
      grid.datasource.filter = conditions;
      grid.datasource.loading = true;
      const cloneConfig = grid.datasource.clone();
      const cloneDs = DatasourceFactory.factory(cloneConfig);
      await cloneDs.get().then(() => {
        const datasource = cloneDs.data;
        Http.post('/modoPreparo/datasource', { datasource }).then((callback) => {
          const texto = callback.data.dataset.result;
          for (let i = 0; i < datasource.length; i += 1) {
            const aux = texto[i];
            texto[i] = datasource[i].TXPREPPRAT;
            datasource[i].TXPREPPRAT = aux;
          }
          grid.datasource.updateData(cloneDs.data);
          grid.datasource = cloneDs;
        });
      });
    } else if (tab.activeTab === 3) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoTipoCard');
      form = Metadata.getInstance<Form>('FORM_TIPOCARD');
      if (form.validate()) {
        conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'CDTIPOCARD',
            operator: '=',
            value: form.value.TIPOCARD.CDTIPOCARD,
          },
        ];
        
        grid.datasource.filter = conditions;
        grid.datasource.loading = true;
        const cloneConfig = grid.datasource.clone();
        const cloneDs = DatasourceFactory.factory(cloneConfig);
        await cloneDs.get().then(() => {
          const datasource = cloneDs.data;
          Http.post('/modoPreparo/datasource', { datasource }).then((callback) => {
            const texto = callback.data.dataset.result;
            for (let i = 0; i < datasource.length; i += 1) {
              const aux = texto[i];
              texto[i] = datasource[i].TXPREPTPCARD;
              datasource[i].TXPREPTPCARD = aux;
            }
            grid.datasource.updateData(cloneDs.data);
            grid.datasource = cloneDs;
          });
        });
      } else this.utilities.showAlert('alert', 'pme::tipoCardapioDeveSerInformado', 'error');
    } else if (tab.activeTab === 4) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoLocal');
      form = Metadata.getInstance<Form>('FORM_LOCAL');
      if (form.validate()) {
        conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'CDFILIAL',
            operator: '=',
            value: form.value.BRANCH_LOCAL.CDFILIAL,
          },
        ];
        
        grid.datasource.filter = conditions;
        grid.datasource.loading = true;
        const cloneConfig = grid.datasource.clone();
        const cloneDs = DatasourceFactory.factory(cloneConfig);
        await cloneDs.get().then(() => {
          const datasource = cloneDs.data;
          Http.post('/modoPreparo/datasource', { datasource }).then((callback) => {
            const texto = callback.data.dataset.result;
            for (let i = 0; i < datasource.length; i += 1) {
              const aux = texto[i];
              texto[i] = datasource[i].TXPREPFILI;
              datasource[i].TXPREPFILI = aux;
            }
            grid.datasource.updateData(cloneDs.data);
            grid.datasource = cloneDs;
          });
        });
      } else this.utilities.showAlert('alert', 'pme::unidDeveSerInformado', 'error');
    } else if (tab.activeTab === 5) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoServico');
      form = Metadata.getInstance<Form>('FORM_SERVICE');
      if (form.validate()) {
        conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'CDFILIAL',
            operator: '=',
            value: form.value.BRANCH_SERVICE.CDFILIAL,
          },
          {
            name: 'CDSERVFILI',
            operator: '=',
            value: form.value.SERVICE.CDSERVFILI,
          },
        ];
        
        grid.datasource.filter = conditions;
        grid.datasource.loading = true;
        const cloneConfig = grid.datasource.clone();
        const cloneDs = DatasourceFactory.factory(cloneConfig);
        await cloneDs.get().then(() => {
          const datasource = cloneDs.data;
          Http.post('/modoPreparo/datasource', { datasource }).then((callback) => {
            const texto = callback.data.dataset.result;
            for (let i = 0; i < datasource.length; i += 1) {
              const aux = texto[i];
              texto[i] = datasource[i].TXPREPSERV;
              datasource[i].TXPREPSERV = aux;
            }
            grid.datasource.updateData(cloneDs.data);
            grid.datasource = cloneDs;
          });
        });
      } else this.utilities.showAlert('alert', 'pme::unidEservDevemSerInformados', 'error');
    }
    
    grid.datasource.loading = false;
    this.buttonDisable();
  }
  
  private async createModalAdd() {
    this.modalAddModoPrep = await ModalService.createFromJson('planejamento/prato/modoPreparo/modals/modo_preparo_add', true);
  }

  public closeModalAdd() {
    this.setFilters();
    this.modalAddModoPrep.hide();
  }
  
  public addRow() {
    if (!this.modalAddModoPrep) {
      this.createModalAdd().then(() => {
        this.modalAddModoPrep.show();
      });
    } else {
      this.modalAddModoPrep.show();
    }
  }
  
  public saveModoPreparo({ component }: IEventParam<Button>) {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    const formAdd = Metadata.getInstance<Form>('formAddModoPreparo');
    const editedRows = formAdd.value;
    
    let form: any = '';
    let filial: any = '';
    let servico: any = '';
    let cardapio: any = '';
    let receita: any = '';

    if (tab.activeTab === 2) {
      receita = 'P';
      editedRows.NRPREPPRAT = editedRows.NRPREP;
      editedRows.IDPREPPRAT = editedRows.IDPREP === undefined ? '2' : editedRows.IDPREP;
      editedRows.NMRPREPPRAT = editedRows.NMRPREP;
      editedRows.TXPREPPRAT = editedRows.TXPREP;
    } else if (tab.activeTab === 3) {
      form = Metadata.getInstance<Form>('FORM_TIPOCARD');
      receita = 'C';
      cardapio = form.value.TIPOCARD.CDTIPOCARD;
      editedRows.NRPREPTPCARD = editedRows.NRPREP;
      editedRows.IDPREPTPCARD = editedRows.IDPREP === undefined ? '2' : editedRows.IDPREP;
      editedRows.NMPREPTPCARD = editedRows.NMRPREP;
      editedRows.TXPREPTPCARD = editedRows.TXPREP;
    } else if (tab.activeTab === 4) {
      form = Metadata.getInstance<Form>('FORM_LOCAL');
      receita = 'L';
      filial = form.value.BRANCH_LOCAL.CDFILIAL;
      editedRows.NRPREPFILI = editedRows.NRPREP;
      editedRows.IDPREPFILI = editedRows.IDPREP === undefined ? '2' : editedRows.IDPREP;
      editedRows.NMRPREPFILI = editedRows.NMRPREP;
      editedRows.TXPREPFILI = editedRows.TXPREP;
    } else if (tab.activeTab === 5) {
      form = Metadata.getInstance<Form>('FORM_SERVICE');
      receita = 'S';
      filial = form.value.BRANCH_SERVICE.CDFILIAL;
      servico = form.value.SERVICE.CDSERVFILI;
      editedRows.NRPREPSERV = editedRows.NRPREP;
      editedRows.IDPREPSERV = editedRows.IDPREP === undefined ? '2' : editedRows.IDPREP;
      editedRows.NMRPREPSERV = editedRows.NMRPREP;
      editedRows.TXPREPSERV = editedRows.TXPREP;
    }
    
    editedRows.CDFILCENTPME = this.appController.getCdCentral();
    editedRows.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
    editedRows.CDPRATO = currentRow.CDPRATO;
    editedRows.TIPO = receita;
    editedRows.CDFILIAL = filial !== '' ? filial : null;
    editedRows.CDSERVFILI = servico !== '' ? servico : null;
    editedRows.CDTIPOCARD = cardapio !== '' ? cardapio : null;
    if (tab.activeTab === 2) editedRows.NRPREPPRAT = editedRows.NRPREPPRAT !== undefined ? editedRows.NRPREPPRAT : '';
    if (tab.activeTab === 3) editedRows.NRPREPTPCARD = editedRows.NRPREPTPCARD !== undefined ? editedRows.NRPREPTPCARD : '';
    if (tab.activeTab === 4) editedRows.NRPREPFILI = editedRows.NRPREPFILI !== undefined ? editedRows.NRPREPFILI : '';
    if (tab.activeTab === 5) editedRows.NRPREPSERV = editedRows.NRPREPSERV !== undefined ? editedRows.NRPREPSERV : '';

    if ((editedRows.TXPREPPRAT !== null && editedRows.TXPREPPRAT !== undefined) || (editedRows.TXPREPFILI !== null && editedRows.TXPREPFILI !== undefined) || (editedRows.TXPREPSERV !== null && editedRows.TXPREPSERV !== undefined)
    || (editedRows.TXPREPTPCARD !== null && editedRows.TXPREPTPCARD !== undefined)) {
      this.utilities.activeLoadingButton(component);
      Http.post('/modoPreparo/save', { editedRows }).then(() => {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.setFilters();
        this.closeModalAdd();
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    } else this.utilities.showAlert('alert', 'pme::textoObrigatorio', 'error');
  }
  
  public saveGrid() {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    let grid: any = '';
    let form: any = '';
    let filial: any = '';
    let servico: any = '';
    let cardapio: any = '';
    let receita: any = '';
    
    if (tab.activeTab === 2) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoPadrao');
      receita = 'P';
    } else if (tab.activeTab === 3) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoTipoCard');
      form = Metadata.getInstance<Form>('FORM_TIPOCARD');
      receita = 'C';
      cardapio = form.value.TIPOCARD.CDTIPOCARD;
    } else if (tab.activeTab === 4) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoLocal');
      form = Metadata.getInstance<Form>('FORM_LOCAL');
      receita = 'L';
      filial = form.value.BRANCH_LOCAL.CDFILIAL;
    } else if (tab.activeTab === 5) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoServico');
      form = Metadata.getInstance<Form>('FORM_SERVICE');
      receita = 'S';
      filial = form.value.BRANCH_SERVICE.CDFILIAL;
      servico = form.value.SERVICE.CDSERVFILI;
    }
    
    const editedRows: IDictionary = grid.getEditedRows();
    editedRows.forEach((row: IDictionary) => {
      row.CDFILCENTPME = this.appController.getCdCentral();
      row.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
      row.CDPRATO = currentRow.CDPRATO;
      row.TIPO = receita;
      row.CDFILIAL = filial !== '' ? filial : null;
      row.CDSERVFILI = servico !== '' ? servico : null;
      row.CDTIPOCARD = cardapio !== '' ? cardapio : null;
    });
    
    if (editedRows.length > 0) {
      Http.post('/modoPreparo/save', { editedRows }).then(() => {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        const toolbarSlot = grid.toolbarSlot;
        toolbarSlot.forEach((e: IComponentRender) => {
          if (e.name === 'cancelButton') e.disabled = true;
          if (e.name === 'saveButton') e.disabled = true;
        });
        this.gridEditController.defaultState(grid);
        this.setFilters();
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public cancelEditing() {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    let grid: any = '';

    if (tab.activeTab === 2) grid = Metadata.getInstance<Grid>('gridModoPreparoPadrao');
    else if (tab.activeTab === 3) grid = Metadata.getInstance<Grid>('gridModoPreparoTipoCard');
    else if (tab.activeTab === 4) grid = Metadata.getInstance<Grid>('gridModoPreparoLocal');
    else if (tab.activeTab === 5) grid = Metadata.getInstance<Grid>('gridModoPreparoServico');
    
    this.gridEditController.cancelEditing({ component: grid });
    const toolbarSlot = grid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = true;
      if (e.name === 'saveButton') e.disabled = true;
    });
  }
  
  public deleteCheckedRows() {
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    let grid: any = '';
    let dataSourceName: any = '';

    if (tab.activeTab === 2) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoPadrao');
      dataSourceName = 'prepprapad';
    } else if (tab.activeTab === 3) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoTipoCard');
      dataSourceName = 'preppratpcard';
    } else if (tab.activeTab === 4) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoLocal');
      dataSourceName = 'prepprafil';
    } else if (tab.activeTab === 5) {
      grid = Metadata.getInstance<Grid>('gridModoPreparoServico');
      dataSourceName = 'preppraser';
    }
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'delete',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.delete(checkedRows, dataSourceName, grid) },
            outline: true,
          },
          {
            name: 'noDelete',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public delete(checkedRows: any, dataSourceName: any, grid: any) {
    const button = Metadata.getInstance<Button>('yesDelete');
    this.utilities.activeLoadingButton(button);
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      grid.cancelEditedRows();
      grid.selectedRows = [];
      DialogService.hide();
      this.utilities.disableLoadingButton(button);
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
      this.setFilters();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
}
